@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2008 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2011 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -74,15 +74,11 @@ callback_hang_test (int converter)
 	SRC_STATE	*src_state ;
 
 	double src_ratio = 1.0 ;
-	long out_count ;
-	int current_out ;
 	int k, error ;
 
 	printf ("\tcallback_hang_test  (%-28s) ....... ", src_get_name (converter)) ;
 	fflush (stdout) ;
 
-	current_out = 0 ;
-
 	/* Perform sample rate conversion. */
 	src_state = src_callback_new (input_callback, converter, 1, &error, NULL) ;
 	if (src_state == NULL)
@@ -93,7 +89,7 @@ callback_hang_test (int converter)
 	for (k = 0 ; k < ARRAY_LEN (pairs) ; k++)
 	{	alarm (1) ;
 		src_ratio = pairs [k].ratio ;
-		out_count = src_callback_read (src_state, src_ratio, pairs [k].count, output) ;
+		src_callback_read (src_state, src_ratio, pairs [k].count, output) ;
 		} ;
 
 	src_state = src_delete (src_state) ;
@@ -107,7 +103,7 @@ callback_hang_test (int converter)
 static void
 alarm_handler (int number)
 {
-	number = 0 ;
+	(void) number ;
 	printf ("\n\n    Error : Hang inside src_callback_read() detected. Exiting!\n\n") ;
 	exit (1) ;
 } /* alarm_handler */
@@ -117,7 +113,7 @@ input_callback (void *cb_data, float **data)
 {
 	static float buffer [20] ;
 
-	cb_data = NULL ;
+	(void) cb_data ;
 	*data = buffer ;
 
 	return ARRAY_LEN (buffer) ;
