@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2002-2008 Erik de Castro Lopo <erikd@mega-nerd.com>
+** Copyright (C) 2002-2011 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ void reverse_data (float *data, int datalen) ;
 
 double calculate_snr (float *data, int len, int expected_peaks) ;
 
-void print_cpu_name (void) ;
+const char * get_cpu_name (void) ;
 
 #if OS_IS_WIN32
 /*
